#pragma once

#include "Object.h"
#include "Array.h"
#include <string>

namespace rei::json {

	class JsonObject;
	class JsonArray;

	enum FieldType {
		Null,
		Number,
		Bool,
		String,
		Object,
		Array
	};

	class FieldValue {

		public:

		FieldValue(const FieldValue& other) noexcept;
		FieldValue& operator=(const FieldValue& other) noexcept;
		FieldValue& operator=(FieldValue&& other) noexcept;
		FieldValue(FieldValue&& other) noexcept;

		FieldValue(FieldType type);
		FieldValue(int value);
		FieldValue(bool value);
		FieldValue(const std::string& value);
		FieldValue(std::string&& value);
		FieldValue(const JsonObject& value);
		FieldValue(JsonObject&& value);
		FieldValue(const JsonArray& value);
		FieldValue(JsonArray&& value) noexcept;

		~FieldValue();

		inline int asNumber() { return number; }
		inline bool asBool() { return boolean; }
		inline std::string& asString() { return string; }
		inline JsonObject& asObject() { return object; }
		inline JsonArray& asArray() { return array; }

		inline bool isNumber() const { return type == FieldType::Number; }
		inline bool isBool() const { return type == FieldType::Bool; }
		inline bool isString() const { return type == FieldType::String; }
		inline bool isObject() const { return type == FieldType::Object; }
		inline bool isArray() const { return type == FieldType::Array; }
		inline bool isNull() const { return type == FieldType::Null; }
		
		private:

		void _copyValues(const FieldValue& other);
		void _moveValues(FieldValue&& other);
		void _destroy();

		FieldType type;
		union {
			int number;
			bool boolean;
			std::string string;
			JsonObject object;
			JsonArray array;
		};
	};
}
